//=============================================================================
// Keke_MenuFreeSpace - メニュー自由欄
// バージョン: 1.0.1
//=============================================================================
// Copyright (c) 2023 ケケー
// Released under the MIT license
// http://opensource.org/licenses/mit-license.php
//=============================================================================

/*:
 * @target MZ
 * @plugindesc メニューに自由欄を追加する
 * @author ケケー
 * @url https://kekeelabo.com
 * 
 * @help
 * 【ver.1.0.1】
 * メニューの各所に何を書いてもいい自由欄を追加する
 * 具体的には以下の3つ
 * ◎ステータス自由欄
 * ◎ステータスヘルプ欄
 * ◎一言メッセージ(UIボタンエリアの自由欄)
 * 自由欄の表示内容はいくつも登録しておき、条件で切り替え可能
 * ストーリー進行と共に切り替えるなど
 * 
 * ● 使い方 ●
 * 
 * ■【機能1】ステータスの自由欄/ヘルプ欄の表示内容を設定
 * 
 * 手順の流れ
 * [1]プラグインパラメータでテキスト登録
 * [2]プラグインパラメータで自由欄パック登録
 * [3]データベースのメモ欄で表示する自由欄パックを指定
 * 
 * ▼[手順1]プラグインパラメータ → テキスト登録　で、
 * 　テキスト内容を作成・登録
 * 
 * ▼[手順2]プラグインパラメータ → 自由欄パック登録　で、
 * 　自由欄パックを作成・登録
 * ◎自由欄パックとはテキストデータの詰め合わせ
 * 　テキストデータを何個でも登録できる
 * ◎テキストデータには「条件」を設定でき、
 * 　リストの上から「最初に条件を満たしたテキストデータ」が選択される
 * 　条件が空の場合も条件を満たしたことになる
 * ◎選択されたテキストデータが実際に自由欄に表示される
 * 　正確には、テキストデータ内の「呼ぶテキスト」により
 * 　呼び出されたテキストが表示される
 * ◎「呼ぶテキスト」には、
 * 　【1】のテキスト登録で作成したテキストのテキスト名を書く
 * 　するとそのテキストを呼び出せる
 * ◎テキストデータのリストはふたつある
 * 　「自由欄テキスト」と「ヘルプテキスト」
 * 　その名の通り自由欄テキストの方は自由欄に表示され、
 * 　ヘルプテキストはヘルプ欄に表示される
 * ◎なお自由欄はステータスの経験値の欄に表示される
 * 　当然、経験値は見れなくなるが、必要なら自由欄の中に、
 * 　現在の経験値: a.currentExp()
 * 　次のレベルまで: a.nextRequiredExp()
 * 　と入力することで表示可能
 * 　
 *
 * ▼[手順3]データベースのメモ欄で表示する自由欄パックを指定
 * ◎アクター、職業、スキル、アイテム、武器、防具、敵キャラ、ステートのメモ欄に
 * 　<ステータス自由欄: パック名>
 * ★例)
 * <ステータス自由欄: プリシア>
 * 　自由欄パック『プリシア』を自由欄に表示する
 * 
 * 
 * ■【機能2】一言メッセージの表示内容を設定
 * 
 * 手順の流れ
 * [1]プラグインパラメータでテキスト登録
 * [2]プラグインパラメータで一言メッセージ登録
 * 
 * ▼[手順1]プラグインパラメータ → テキスト登録　で、
 * 　テキスト内容を作成・登録
 * 
 * ▼[手順2]プラグインパラメータ → 一言メッセージ登録　で、
 * 　一言メッセージを作成・登録
 * ◎一言メッセージとはUIボタンエリア欄に表示されるテキストのこと
 * ◎一言メッセージ登録はテキストデータの詰め合わせ
 * 　テキストデータを何個でも登録できる
 * ◎テキストデータには「条件」を設定でき、
 * 　リストの中から「条件を満たした全てのテキストデータ」が選択される
 * 　条件が空の場合も条件を満たしたことになる
 * ◎選択されたテキストデータが順番に表示される
 * 　メニューを開くたびに次のテキストに変わる。最後までいったら最初に戻る
 * ◎正確には、テキストデータ内の「呼ぶテキスト」により
 * 　呼び出されたテキストが表示される
 * ◎「呼ぶテキスト」には、
 * 　【1】のテキスト登録で作成したテキストのテキスト名を書く
 * 　するとそのテキストを呼び出せる
 * ◎一言メッセージの内容を変更した際には、
 * 　表示順を初期化することをオススメ
 * 　プラグインコマンド → 一言メッセージ初期化　で初期化できる
 * 
 * 
 * ■【機能3】レイアウトを設定
 * 
 * ◎ステータス自由欄のレイアウトなら、
 * プラグインパラメータ → ステータス自由欄
 * 
 * ◎ステータスヘルプ欄のレイアウトなら、
 * プラグインパラメータ → ステータスヘルプ
 * 
 * ◎一言メッセージのレイアウトなら、
 * プラグインパラメータ → 一言メッセージ
 * 
 * 
 * ● 利用規約 ●
 * MITライセンスのもと、自由に使ってくれて大丈夫です
 * 
 * 
 * 
 * Add free space where you can write anything you want 
 *  in various parts of the menu
 * Specifically, the following three
 * ◎ StatusFreeSpace
 * ◎ StatusHelpSpace
 * ◎ ShortMessage (free space in UI button area)
 * You can register a number of display contents in the free space 
 *  and switch them according to conditions.
 * Switch as the story progresses, etc.
 *
 * ● How to use ●
 *
 * ■ [Function 1] Set the display contents of the status free space/help space
 * 
 * Procedure flow
 * [1] TextMaking with plugin parameters
 * [2] FreePackMaking with plugin parameters
 * [3] Specify the text pack to be displayed in the database memo field
 *
 * ▼[Step 1] Plugin parameters → TextMaking,
 * Create and register text content
 *
 * ▼[Step 2] Plugin parameters → FreePackMaking,
 * Create and register free packs
 * ◎ What is a free pack? An assortment of text data
 * Any number of text data can be registered
 * ◎ “Condition” can be set for text data,
 * "The first text data that satisfies the condition" is selected 
 *  from the top of the list.
 * Even if the condition is empty, the condition is met
 * ◎ The selected text data is actually displayed in the free field.
 * To be exact, it depends on the "CallText" in the text data.
 * The called text is displayed
 * ◎ “CallText” includes
 * Write the text name of the text maked in [1] text making
 * Then you can call the text
 * ◎ There are two lists of text data.
 * "FreeSpaceText" and "HelpText"
 * As the name suggests, free space text is displayed in the free space,
 * Help text is displayed in the help space
 * ◎ The free space is displayed in the experience value space of the status.
 * Of course, you will not be able to see your experience points, 
 * but if you wish, you can enter them in the free space.
 * Total exp: a.currentExp()
 * Next level exp: a.nextRequiredExp()
 * Can be displayed by entering
 *
 * ▼[Step 3] Specify the free pack to be displayed 
 *  in the memo field of the database
 * ◎ In the memo field for actors, occupations, skills, items, weapons, armor, 
 *  enemy characters, and states.
 * <statusFreeSpace: packName>
 * ★Example)
 * <statusFreeSpace: pricia>
 * Display the free pack "pricia" in the free space
 *
 *
 * ■[Function 2] Set the display content of the short message
 * 
 * Procedure flow
 * [1] TextMaking with plugin parameters
 * [2] ShortMessageMaking width plugin parameters
 *
 * ▼[Step 1] Plugin parameters → TextMaking,
 * Create and register text content
 *
 * ▼[Step 2] Plugin parameters → ShortMessageMaking,
 * Create and register a short message
 * ◎ Short message is the text displayed from the UI button area column.
 * ◎ Short message making is an assortment of text data
 * Any number of text data can be registered
 * ◎ “Conditions” can be set for text data,
 * "All text data that meets the conditions" is selected from the list
 * Even if the condition is empty, the condition is met
 * ◎ Selected text data will be displayed in order
 * The text changes to the following every time the menu is opened
 * When you get to the end, go back to the beginning
 * ◎ Accurately, depend on the "CallText" in the text data
 * The called text is displayed
 * ◎ “CallText” includes
 * Write the text name of the maked text in [1] text registration
 * Then you can call the text
 * ◎ When changing the content of a short message,
  * It is recommended to initialize the display order.
  * Can be initialized with plugin command → InitShortMessage
 *
 *
 * ■[Function 3] Set layout
 *
 * ◎ For the status free space layout,
 * Plugin parameters → StatusFreeSpace
 *
 * ◎ For the layout of the status help space,
 * Plugin parameters → StatusHelp
 *
 * ◎ For a short message layout,
 * Plugin parameters → ShortMessage
 *
 *
 * ● Terms of Use ●
 * You can use it freely under the MIT license.
 * 
 *
 *
 * @param 自由欄パック登録
 * @desc freePackMaking ステータス自由欄に表示するテキスト群を登録する。メモ欄から<ステータス自由欄: パック名>で呼び出せる
 * @type struct<freePackMake>[]
 * @default 
 * 
 * @param 自由欄パック_1
 * @parent 自由欄パック登録
 * @desc freePackMaking ステータス自由欄に表示するテキスト群を登録する。メモ欄から<ステータス自由欄: パック名>で呼び出せる
 * @type struct<freePackMake>[]
 * @default ["{\"パック名\":\"サンプル\",\"自由欄テキスト\":\"[\\\"{\\\\\\\"呼ぶテキスト\\\\\\\":\\\\\\\"サンプル/固有スキル\\\\\\\",\\\\\\\"条件\\\\\\\":\\\\\\\"\\\\\\\"}\\\",\\\"{\\\\\\\"呼ぶテキスト\\\\\\\":\\\\\\\"サンプル/日記\\\\\\\",\\\\\\\"条件\\\\\\\":\\\\\\\"\\\\\\\"}\\\",\\\"{\\\\\\\"呼ぶテキスト\\\\\\\":\\\\\\\"サンプル/プロフィール\\\\\\\",\\\\\\\"条件\\\\\\\":\\\\\\\"\\\\\\\"}\\\",\\\"{\\\\\\\"呼ぶテキスト\\\\\\\":\\\\\\\"追加能力値\\\\\\\",\\\\\\\"条件\\\\\\\":\\\\\\\"\\\\\\\"}\\\"]\",\"ヘルプテキスト\":\"[\\\"{\\\\\\\"呼ぶテキスト\\\\\\\":\\\\\\\"サンプル/プロフィールh\\\\\\\",\\\\\\\"条件\\\\\\\":\\\\\\\"\\\\\\\"}\\\"]\"}"]
 * 
 * @param 自由欄パック_2
 * @parent 自由欄パック登録
 * @desc freePackMaking ステータス自由欄に表示するテキスト群を登録する。メモ欄から<ステータス自由欄: パック名>で呼び出せる
 * @type struct<freePackMake>[]
 * @default 
 * 
 * @param 自由欄パック_3
 * @parent 自由欄パック登録
 * @desc freePackMaking ステータス自由欄に表示するテキスト群を登録する。メモ欄から<ステータス自由欄: パック名>で呼び出せる
 * @type struct<freePackMake>[]
 * @default 
 * 
 * @param 自由欄パック_4
 * @parent 自由欄パック登録
 * @desc freePackMaking ステータス自由欄に表示するテキスト群を登録する。メモ欄から<ステータス自由欄: パック名>で呼び出せる
 * @type struct<freePackMake>[]
 * @default 
 * 
 * @param 自由欄パック_5
 * @parent 自由欄パック登録
 * @desc freePackMaking ステータス自由欄に表示するテキスト群を登録する。メモ欄から<ステータス自由欄: パック名>で呼び出せる
 * @type struct<freePackMake>[]
 * @default 
 * 
 * @param 一言メッセージ登録
 * @desc shortMessageMaking UIボタンエリアに表示される一言メッセージを登録する。条件を満たした物が順に表示される
 * @type struct<textData>[]
 * @default 
 * 
 * @param 一言メッセージ_1
 * @parent 一言メッセージ登録
 * @desc shortMessageMaking UIボタンエリアに表示される一言メッセージを登録する。条件を満たした物が順に表示される
 * @type struct<textData>[]
 * @default ["{\"呼ぶテキスト\":\"サンプル/一言_1\",\"条件\":\"\"}","{\"呼ぶテキスト\":\"サンプル/一言_2\",\"条件\":\"\"}","{\"呼ぶテキスト\":\"サンプル/一言_3\",\"条件\":\"\"}"]
 * 
 * @param 一言メッセージ_2
 * @parent 一言メッセージ登録
 * @desc shortMessageMaking UIボタンエリアに表示される一言メッセージを登録する。条件を満たした物が順に表示される
 * @type struct<textData>[]
 * @default 
 * 
 * @param 一言メッセージ_3
 * @parent 一言メッセージ登録
 * @desc shortMessageMaking UIボタンエリアに表示される一言メッセージを登録する。条件を満たした物が順に表示される
 * @type struct<textData>[]
 * @default 
 * 
 * @param 一言メッセージ_4
 * @parent 一言メッセージ登録
 * @desc shortMessageMaking UIボタンエリアに表示される一言メッセージを登録する。条件を満たした物が順に表示される
 * @type struct<textData>[]
 * @default 
 * 
 * @param 一言メッセージ_5
 * @parent 一言メッセージ登録
 * @desc shortMessageMaking UIボタンエリアに表示される一言メッセージを登録する。条件を満たした物が順に表示される
 * @type struct<textData>[]
 * @default 
 * 
 * @param テキスト登録
 * @desc textMaking テキスト内容を登録する。自由欄パック登録/一言メッセージ登録の「呼び出すテキスト」から呼び出せる
 * @type struct<textMake>[]
 * @default 
 * 
 * @param テキスト_1
 * @parent テキスト登録
 * @desc textMaking テキスト内容を登録する。自由欄パック登録/一言メッセージ登録の「呼び出すテキスト」から呼び出せる
 * @type struct<textMake>[]
 * @default ["{\"テキスト名\":\"◆自由欄テキスト\",\"テキスト\":\"\"}","{\"テキスト名\":\"■固有スキル\",\"テキスト\":\"\"}","{\"テキスト名\":\"サンプル/固有スキル\",\"テキスト\":\"\\\\fs[+2]\\\\c[24]固有スキル\\\\cl\\n\\\\c[17]ブラッドスティア:\\\\cl HPMPTPを吸収する謎の剣技\\n\\\\c[17]ブッ殺！！！:\\\\cl 何か知らんがすごいダメージ\\n\\\\c[26]剣の才能S:\\\\cl 剣技の性能が2倍\\n\\\\c[6]あたしは無敵:\\\\cl 防御の性能がとんでもなくアップ\\n\\\\c[27]人任せの極意:\\\\cl パスの性能が大幅アップ \"}","{\"テキスト名\":\"■日記\",\"テキスト\":\"\"}","{\"テキスト名\":\"サンプル/日記\",\"テキスト\":\"\\\\fs[18]\\\\c[27]プリシアちゃんの日記その1\\\\c[0]\\n突然だが、あたしは罪深き存在だ\\nなぜならヒロインとして魅力的すぎるから\\nMZには他にもデフォキャラがいるが、\\nあたしのせいで存在感消えてしまっている\\nあたしがまばゆすぎるから、可愛すぎるから！\\nなんて罪深い…。だからせめて、MZの発展に\\n全力を尽くす！ それがあたしの使命だと思う\"}","{\"テキスト名\":\"■プロフィール\",\"テキスト\":\"\"}","{\"テキスト名\":\"サンプル/プロフィール\",\"テキスト\":\"\\\\c[16]名前:\\\\cl  田中 可憐(たなか かれん)\\n\\\\c[16]年齢:\\\\cl  18歳　\\\\c[16]出身地:\\\\cl  日本/東京\\n\\\\c[16]身長:\\\\cl  170cm　\\\\c[16]体重:\\\\cl  63kg\\n\\\\c[16]顔面偏差値:\\\\cl  80　\\\\c[16]職業:\\\\cl  家事手伝い\\n\\\\c[16]才能:\\\\cl  剣技S 保身SSS おねだりA\\n\\\\c[16]好きな物:\\\\cl  あたしの偉大なる人生\\n\\\\c[16]嫌いな物:\\\\cl  地道な努力\\n\\\\c[16]目標:\\\\cl  あたしのために働く彼氏募集中！\\n\"}","{\"テキスト名\":\"■追加能力値\",\"テキスト\":\"\"}","{\"テキスト名\":\"追加能力値\",\"テキスト\":\"\\\\c[16]命中率:\\\\cl  \\\\sc[a.hit*100]%　\\\\c[16]回避率:\\\\cl  \\\\sc[a.eva*100]%\\n\\\\c[16]会心率:\\\\cl  \\\\sc[a.cri*100]%　\\\\c[16]会心回避率:\\\\cl  \\\\sc[a.cev*100]%\\n\\\\c[16]魔法回避率:\\\\cl  \\\\sc[a.mev*100]%　\\\\c[16]魔法反射率:\\\\cl  \\\\sc[a.mrf*100]%\\n\\\\c[16]反撃率:\\\\cl  \\\\sc[a.cnt*100]%　\\\\c[16]HP再生率:\\\\cl  \\\\sc[a.hrg*100]%\\n\\\\c[16]MP再生率:\\\\cl  \\\\sc[a.mrg*100]%　\\\\c[16]TP再生率:\\\\cl  \\\\sc[a.trg*100]%\\n\\\\c[16]狙われ率:\\\\cl  \\\\sc[a.tgr*100]%　\\\\c[16]MP消費率:\\\\cl  \\\\sc[a.mcr*100]%\\n\\\\c[16]防御効果率:\\\\cl  \\\\sc[a.grd*100]%　\\\\c[16]回復効果率:\\\\cl  \\\\sc[a.rec*100]%\\n\\\\c[16]合計経験値:\\\\cl  \\\\sc[a.currentExp()]\\\\c[16]　\\\\c[16]次まで:\\\\cl  \\\\sc[a.nextRequiredExp()]\"}"]
 * 
 * @param テキスト_2
 * @parent テキスト登録
 * @desc textMaking テキスト内容を登録する。自由欄パック登録/一言メッセージ登録の「呼び出すテキスト」から呼び出せる
 * @type struct<textMake>[]
 * @default ["{\"テキスト名\":\"◆ヘルプ欄テキスト\",\"テキスト\":\"\"}","{\"テキスト名\":\"■プロフィールh\",\"テキスト\":\"\"}","{\"テキスト名\":\"サンプル/プロフィールh\",\"テキスト\":\"\\\\c[16]名前:\\\\cl 田中 可憐(たなか かれん)　\\\\c[16]年齢:\\\\cl 18歳　\\\\c[16]身長:\\\\cl 170cm　\\\\c[16]体重:\\\\cl 63kg\\n\\\\c[16]才能:\\\\cl 剣技S 自己保身SSS おねだりA　\\\\c[16]顔面偏差値:\\\\cl 80　\\\\c[16]出身地:\\\\cl 日本/東京\\n何事も人任せで利益だけを享受する者。あたしの辞書に労苦の二文字はない\\n\\n\"}"]
 * 
 * @param テキスト_3
 * @parent テキスト登録
 * @desc textMaking テキスト内容を登録する。自由欄パック登録/一言メッセージ登録の「呼び出すテキスト」から呼び出せる
 * @type struct<textMake>[]
 * @default ["{\"テキスト名\":\"◆一言メッセージ\",\"テキスト\":\"\"}","{\"テキスト名\":\"サンプル/一言_1\",\"テキスト\":\"ケケーラボの他のプラグインもよろしくね\"}","{\"テキスト名\":\"サンプル/一言_2\",\"テキスト\":\"メニュースキン、メニュー快適化、メニュー自由欄の3つのプラグインで\"}","{\"テキスト名\":\"サンプル/一言_3\",\"テキスト\":\"メニュー画面を見た目も機能も大幅パワーアップだ！！\"}"]
 * 
 * @param テキスト_4
 * @parent テキスト登録
 * @desc textMaking テキスト内容を登録する。自由欄パック登録/一言メッセージ登録の「呼び出すテキスト」から呼び出せる
 * @type struct<textMake>[]
 * @default 
 * 
 * @param テキスト_5
 * @parent テキスト登録
 * @desc textMaking テキスト内容を登録する。自由欄パック登録/一言メッセージ登録の「呼び出すテキスト」から呼び出せる
 * @type struct<textMake>[]
 * @default 
 * 
 * @param テキスト_6
 * @parent テキスト登録
 * @desc textMaking テキスト内容を登録する。自由欄パック登録/一言メッセージ登録の「呼び出すテキスト」から呼び出せる
 * @type struct<textMake>[]
 * @default 
 * 
 * @param テキスト_7
 * @parent テキスト登録
 * @desc textMaking テキスト内容を登録する。自由欄パック登録/一言メッセージ登録の「呼び出すテキスト」から呼び出せる
 * @type struct<textMake>[]
 * @default 
 * 
 * @param テキスト_8
 * @parent テキスト登録
 * @desc textMaking テキスト内容を登録する。自由欄パック登録/一言メッセージ登録の「呼び出すテキスト」から呼び出せる
 * @type struct<textMake>[]
 * @default 
 * 
 * @param テキスト_9
 * @parent テキスト登録
 * @desc textMaking テキスト内容を登録する。自由欄パック登録/一言メッセージ登録の「呼び出すテキスト」から呼び出せる
 * @type struct<textMake>[]
 * @default 
 * 
 * @param レイアウト設定
 * 
 * @param ステータス自由欄
 * @parent レイアウト設定
 * @desc statsuFreeSpace_layout ステータス自由欄のレイアウト設定
 * @type struct<layoutFreeSpaceStatus>
 * @default {"位置X":"392","位置Y":"-10","文字サイズ":"18","文字縁取り幅":"5","文字色":"","バック":"","バック表示":"true","バック色":"0, 0, 0, 0.3","バック丸み":"20","バック余白X":"0","バック余白Y":"0"}
 * 
 * @param ステータスヘルプ
 * @parent レイアウト設定
 * @desc statsuHelp_layout ステータスヘルプ欄のレイアウト設定
 * @type struct<layoutStatusHelp>
 * @default {"位置X":"0","位置Y":"-8","文字サイズ":"20","文字縁取り幅":"5","文字色":"","バック":"","バック表示":"true","バック色":"0, 0, 0, 0.3","バック丸み":"20","バック余白X":"0","バック余白Y":"0"}
 *
 * @param 一言メッセージ
 * @parent レイアウト設定
 * @desc shortMessage_layout 一言メッセージのレイアウト設定
 * @type struct<layoutShortMessage>
 * @default {"位置X":"0","位置Y":"-4","文字サイズ":"26","文字縁取り幅":"6","文字色":"","表示しないシーン":"[\"Options\",\"GameEnd\"]","表示順":"リスト順","バック":"","バック表示":"false","バック色":"0, 0, 0, 0.3","バック丸み":"20","バック余白X":"0","バック余白Y":"0"}
 *
 *
 *
 * @command 一言メッセージ初期化
 * @desc initShortMessage 一言メッセージの表示順を最初に戻す
 *
 * @arg 
 * @desc 
 * @default 
 */



//==================================================
/*~struct~freePackMake:
//==================================================
 * @param パック名
 * @desc packName 自由欄パックの名前。メモ欄からの呼び出しに使う
 *
 * @param 自由欄テキスト
 * @desc freeTextList 自由欄に表示するテキストのリスト。上から条件を満たしたものを採用する
 * @type struct<textData>[]
 * 
 * @param ヘルプテキスト
 * @desc helpTextList ヘルプ欄に表示するテキストのリスト。上から条件を満たしたものを採用する
 * @type struct<textData>[]
 */



//==================================================
/*~struct~textMake:
//==================================================
 * @param テキスト名
 * @desc textName テキストの名前。自由欄パックからの呼び出しに使う
 *
 * @param テキスト
 * @desc test 表示するテキスト内容
 * @type multiline_string
 */


//==================================================
/*~struct~textData:
//==================================================
 * @param 呼ぶテキスト
 * @desc callText 呼び出すテキスト。テキスト登録したテキスト名を書く
 * 
 * @param 条件
 * @desc condition 表示する条件。javaScript式で記述。a:アクター v:変数 s:スイッチ
 * @type multiline_string
 */



//==================================================
/*~struct~freeText:
//==================================================
 * @param タグ
 * @desc 何を書いてもいい欄
 *
 * @param フリーテキスト
 * @desc freeText 自由欄に表示するテキスト
 * @type multiline_string
 * 
 * @param 条件
 * @desc condition 表示する条件。javaScript式で記述。a:アクター v:変数 s:スイッチ
 * @type multiline_string
 */



//==================================================
/*~struct~helpText:
//==================================================
 * @param タグ
 * @desc 何を書いてもいい欄
 * 
 * @param ヘルプテキスト
 * @desc helpText ヘルプ欄に表示するテキスト
 * @type multiline_string
 * 
 * @param 条件
 * @desc condition 表示する条件。javaScript式で記述。a:アクター v:変数 s:スイッチ
 * @type multiline_string
 */



//==================================================
/*~struct~layoutFreeSpaceStatus:
//==================================================
 * @param 位置X
 * @desc posX ステータス自由欄の文字描画開始X位置。5 ならウインドウ左端から 5ピクセル。基本 392
 * @default 392
 * 
 * @param 位置Y
 * @desc posY ステータス自由欄の文字描画開始Y位置。5 ならウインドウ上端から 5ピクセル。基本 -10
 * @default -10
 * 
 * @param 文字サイズ
 * @desc fontSize ステータス自由欄の文字サイズ。空欄なら標準サイズ。+1 で標準サイズ + 1、-1 で標準サイズ - 1。基本 18
 * @default 18
 * 
 * @param 文字縁取り幅
 * @desc fontOutWidth ステータス自由欄の文字縁取り幅。空欄なら標準に合わせる。基本 5
 * @default 5
 * 
 * @param 文字色
 * @desc textColor ステータス自由欄の文字色。赤, 緑, 青。各0～255。空欄なら標準に合わせる。基本 空欄
 * @default 
 * 
 * @param バック
 * 
 * @param バック表示
 * @parent バック
 * @desc ステータス自由欄に背景部分を表示する
 * @type boolean
 * @default true
 * 
 * @param バック色
 * @parent バック
 * @desc 背景部分の色色。赤, 緑, 青, 濃度。色0～255、濃度0～1。空欄なら標準に合わせる。基本 0, 0, 0, 0.3
 * @default 0, 0, 0, 0.3
 * 
 * @param バック丸み
 * @parent バック
 * @desc 背景部分の角の丸み。5 なら 5ピクセル 丸まる。基本 20 
 * @default 20
 * 
 * @param バック余白X
 * @parent バック
 * @desc 背景部分のX方向の余白。5 なら 5ピクセル。基本 0
 * @default 0
 * 
 * @param バック余白Y
 * @parent バック
 * @desc 背景部分のY方向の余白。5 なら 5ピクセル。基本 0
 * @default 0
 */



//==================================================
/*~struct~layoutStatusHelp:
//==================================================
 * @param 位置X
 * @desc posX ステータスヘルプの文字描画開始X位置。5 ならウインドウ左端から 5ピクセル。基本 0
 * @default 0
 * 
 * @param 位置Y
 * @desc posY ステータスヘルプの文字描画開始Y位置。5 ならウインドウ上端から 5ピクセル。基本 -8
 * @default -8
 * 
 * @param 文字サイズ
 * @desc fontSize ステータスヘルプの文字サイズ。空欄なら標準サイズ。+1 で標準サイズ + 1、-1 で標準サイズ - 1。基本 20
 * @default 20
 * 
 * @param 文字縁取り幅
 * @desc fontOutWidth ステータスヘルプの文字縁取り幅。空欄なら標準に合わせる。基本 5
 * @default 5
 * 
 * @param 文字色
 * @desc textColor ステータスヘルプの文字色。赤, 緑, 青。各0～255。空欄なら標準に合わせる。基本 空欄
 * @default 
 * 
 * @param バック
 * 
 * @param バック表示
 * @parent バック
 * @desc ステータス自由欄に背景部分を表示する
 * @type boolean
 * @default true
 * 
 * @param バック色
 * @parent バック
 * @desc 背景部分の色色。赤, 緑, 青, 濃度。色0～255、濃度0～1。空欄なら標準に合わせる。基本 0, 0, 0, 0.3
 * @default 0, 0, 0, 0.3
 * 
 * @param バック丸み
 * @parent バック
 * @desc 背景部分の角の丸み。5 なら 5ピクセル 丸まる。基本 20 
 * @default 20
 * 
 * @param バック余白X
 * @parent バック
 * @desc 背景部分のX方向の余白。5 なら 5ピクセル。基本 0
 * @default 0
 * 
 * @param バック余白Y
 * @parent バック
 * @desc 背景部分のY方向の余白。5 なら 5ピクセル。基本 0
 * @default 0
 */



//==================================================
/*~struct~layoutShortMessage:
//==================================================
 * @param 位置X
 * @desc posX 一言メッセージの文字描画開始X位置。5 ならウインドウ左端から 5ピクセル。基本 0
 * @default 0
 * 
 * @param 位置Y
 * @desc posY 一言メッセージの文字描画開始Y位置。5 ならウインドウ上端から 5ピクセル。基本 -4
 * @default -4
 * 
 * @param 文字サイズ
 * @desc fontSize 一言メッセージの文字サイズ。空欄なら標準サイズ。+1 で標準サイズ + 1、-1 で標準サイズ - 1。基本 26
 * @default 26
 * 
 * @param 文字縁取り幅
 * @desc fontOutWidth 一言メッセージの文字縁取り幅。空欄なら標準に合わせる。基本 6
 * @default 6
 * 
 * @param 文字色
 * @desc textColor 一言メッセージの文字色。赤, 緑, 青。各0～255。空欄なら標準に合わせる。基本 空欄
 * @default
 * 
 * @param 表示しないシーン
 * @desc hideScene 一言メッセージを表示しないシーン。オプションシーンなら Options。シーン名の頭の Scene_ は不要
 * @type string[]
 * @default ["Options","GameEnd"]
 * 
 * @param メインメニューのみ
 * @desc showOnlyMainMenu メインメニュー画面でのみ一言メッセージを表示する。基本 false
 * @type boolean
 * @default false
 * 
 * @param 表示順
 * @desc shorOrder メッセージが複数ある時どのような順番で表示するか
 * @type select
 * @option リスト順
 * @option ランダム
 * @default リスト順
 * 
 * @param UIボタンエリア高さ
 * @desc uiButtonAreaHeight 一言メッセージが表示されるUIボタンエリアの高さ。50 なら 50ピクセル 基本 52
 * @default 52
 * 
 * @param 行間
 * @desc lineSpace 一言メッセージが複数行にまたがる時の行間。5 なら 5ビクセル。基本 4
 * @default 4
 * 
 * @param バック
 * 
 * @param バック表示
 * @parent バック
 * @desc ステータス自由欄に背景部分を表示する
 * @type boolean
 * @default false
 * 
 * @param バック色
 * @parent バック
 * @desc 背景部分の色色。赤, 緑, 青, 濃度。色0～255、濃度0～1。空欄なら標準に合わせる。基本 0, 0, 0, 0.3
 * @default 0, 0, 0, 0.3
 * 
 * @param バック丸み
 * @parent バック
 * @desc 背景部分の角の丸み。5 なら 5ピクセル 丸まる。基本 20 
 * @default 20
 * 
 * @param バック余白X
 * @parent バック
 * @desc 背景部分のX方向の余白。5 なら 5ピクセル。基本 0
 * @default 0
 * 
 * @param バック余白Y
 * @parent バック
 * @desc 背景部分のY方向の余白。5 なら 5ピクセル。基本 0
 * @default 0
 */
 
 
 
(() => {
    //- プラグイン名
    const pluginName = document.currentScript.src.match(/^.*\/(.*).js$/)[1];
    
    
    
    //==================================================
    //--  文字列オート変換 /ベーシック
    //==================================================
    
    // 文字列のハッシュ化
    function strToHash(str) {
        if (!str || !str.length) { return {}; }
        let hash = {};
        const strs = JSON.parse(str);
        let val = null;
        let val2 = null;
        for (let key in strs) {
            val = strs[key];
            if (!key || !val) { continue; }
            val2 = strToAuto(val, key);
            hash[key] = val2;
        }
        return hash;
    };
    
    
    // 文字列のリスト化
    function strToList(str) {
        if (!str || !str.length) { return []; }
        let array = JSON.parse(str);
        return array.map((val, i) => {
            return strToAuto(val);
        }, this);
    };
    
    
    // 文字列の自動処理
    function strToAuto(val, key = "") {
        let val2 = null;
        let match = null;
        let end = false;
        if (!end) {
            if (val[0] == "{") {
                val2 = strToHash(val);
                end = true;
            }
        }
        if (!end) {
            if (val[0] == "[") {
                val2 = strToList(val);
                end = true;
            }
        }
        if (!end) { val = val + ","; }
        if (!end) {
            match = val.match(/^\s*(-?\d+,\s*-?\d+,\s*-?\d+,?\s*-?\d*\.?\d*)\s*,$/);
            if (match && !val.match(/[^\d\.\-,\s]/)) {
                if (key.match(/(カラー|色|塗り)/) && !key.includes("トーン") && !key.includes("ブレンド") && !key.includes("配色") && !key.includes("着色") &&  !key.includes("フラッシュ") && !key.includes("チェンジ") &&  !key.includes("選択")) {
                    val2 = "rgba(" +  match[1] + ")";
                } else {
                    val2 = JSON.parse("[" +  match[1] + "]");
                }
                end = true;
            }
        }
        if (!end) {
            match = val.match(/(-?\d+\.?\d*),\s*/g);
            if (match && match.length >= 2 && !val.match(/[^\d\.\-,\s]/)) {
                val2 = JSON.parse("[" + match.reduce((r, s) => r + s).replace(/,$/, "") + "]");
                end = true;
            }
        }
        if (!end) {
            match = val.match(/^(true|false)\s*,/);
            if (match) {
                val2 = match[1] == "true" ? true : false;
                end = true;
            }
        }
        if (!end) {
            match = val.match(/^(-?\d+\.?\d*)\s*,/);
            if (match && !val.match(/[^\d\.\-,\s]/)) {
                val2 = Number(match[1]); end = true;
                end = true;
            }
        }
        if (!end) {
            match = val.match(/^.+,.+/);
            if (match) {
                val2 = val.replace(/\s/g, "").split(",").filter(v => v);
                end = true;
            }
        }
        if (!end) {
            if (val[0] == "\"") { val = val.slice(1); }
            val2 = val.slice(0, -1);
        }
        return val2;
    };



    //==================================================
    //--  パラメータ受け取り
    //==================================================
    
    let parameters = PluginManager.parameters(pluginName);
    
    //- データ登録
    const keke_freePackMaking = compileParameters(parameters, "自由欄パック登録", "自由欄パック_");
    const keke_shortMessageMaking = compileParameters(parameters, "一言メッセージ登録", "一言メッセージ_");
    const keke_textMaking =compileParameters(parameters, "テキスト登録", "テキスト_");

    //- レイアウト設定
    const keke_freeSpaceStatus = strToHash(parameters["ステータス自由欄"]);
    const keke_StatusHelp = strToHash(parameters["ステータスヘルプ"]); 
    const keke_shortMessage = strToHash(parameters["一言メッセージ"]); 

    parameters = null;

    //- パラメータの合体
    function compileParameters(parameters, word1, word2) {
        let list = strToList(parameters[word1]);
        let i = 1;
        while (parameters[word2 + i]) {
            const addList = strToList(parameters[word2 + i]);
            list = [...list, ...addList];
            i++;
        }
        return list
    };
    
    
    
    //==================================================
    //--  プラグインコマンド
    //==================================================
    
    //- 一言メッセージ初期化
    PluginManager.registerCommand(pluginName, "一言メッセージ初期化", args => {
        $gameSystem._shortMessageNumberKeStfs = 0;
    });
    
    
    
    //==================================================
    //--  共通開始
    //==================================================
    
    
    
    //==================================================
    //--  共通更新
    //==================================================
    
    
    
    //==================================================
    //--  共通終了
    //==================================================
    
    

    //==================================================
    //--  追加コンテンツスプライト-ステータス
    //==================================================

    //- 追加コンテンツスプライトの形成(コア追加)
    const _Window_Status_createContentsSprite =  Window_Status.prototype._createContentsSprite;
    Window_Status.prototype._createContentsSprite = function() {
        this._contentsSpriteKeStfs = new Sprite();
        this.addChild(this._contentsSpriteKeStfs);
        //Window.prototype._createContentsSprite.call(this);
        _Window_Status_createContentsSprite.apply(this);
    };


    //- 追加コンテンツスプライトの更新(コア追加)
    const _Window_Status_updateContents =  Window_Status.prototype._updateContents;
    Window_Status.prototype._updateContents = function() {
        //Window.prototype._updateContents.call(this);
        _Window_Status_updateContents.apply(this);
        const bitmap = this._contentsSpriteKeStfs.bitmap;
        if (bitmap) {
            this._contentsSpriteKeStfs.setFrame(0, 0, bitmap.width, bitmap.height);
        }
    };


    //- 追加コンテンツの形成(コア追加)
    const _Window_Status_createContents = Window_Status.prototype.createContents;
    Window_Status.prototype.createContents = function() {
        Window_Base.prototype.createContents.call(this);
        _Window_Status_createContents.apply(this);
        const width = Graphics.width;
        const height = Graphics.height;
        this._contentsSpriteKeStfs.bitmap = new Bitmap(width, height);
    };


    //- 追加コンテンツの破棄(コア追加)
    const _Window_Status_destroyContents =  Window_Status.prototype.destroyContents;
    Window_Status.prototype.destroyContents = function() {
        //Window_Base.prototype.destroyContents.call(this);
        _Window_Status_destroyContents.apply(this);
        const contetnsEx = this._contentsSpriteKeStfs;
        if (contetnsEx && contetnsEx.bitmap) {
            contetnsEx.bitmap.destroy();
        }
    };


    //- 追加コンテンツのクリア(コア追加)
    const _Window_Status_paint = Window_Status.prototype.paint;
    Window_Status.prototype.paint = function() {
        const contetnsEx = this._contentsSpriteKeStfs;
        if (contetnsEx && contetnsEx.bitmap) {
            contetnsEx.bitmap.clear();
        }
        //Window_Selectable.prototype.paint.call(this);
        _Window_Status_paint.apply(this);
    };



    //==================================================
    //--  追加コンテンツスプライト-ヘルプ
    //==================================================

    //- 追加コンテンツスプライトの形成(コア追加)
    const _Window_Help_createContentsSprite = Window_Help.prototype._createContentsSprite;
    Window_Help.prototype._createContentsSprite = function() {
        if (SceneManager._scene.constructor.name == "Scene_Status") {
            this._contentsSpriteKeStfs = new Sprite();
            this.addChild(this._contentsSpriteKeStfs);
        }
        //Window.prototype._createContentsSprite.call(this);
        _Window_Help_createContentsSprite.apply(this);
    };


    //- 追加コンテンツスプライトの更新(コア追加)
    const _Window_Help_updateContents = Window_Help.prototype._updateContents;
    Window_Help.prototype._updateContents = function() {
        //Window.prototype._updateContents.call(this);
        _Window_Help_updateContents.apply(this);
        if (SceneManager._scene.constructor.name == "Scene_Status") {
            const bitmap = this._contentsSpriteKeStfs.bitmap;
            if (bitmap) {
                this._contentsSpriteKeStfs.setFrame(0, 0, bitmap.width, bitmap.height);
            }
        }
    };


    //- 追加コンテンツの形成(コア追加)
    const _Window_Help_createContents = Window_Help.prototype.createContents;
    Window_Help.prototype.createContents = function() {
        Window_Base.prototype.createContents.call(this);
        _Window_Help_createContents.apply(this);
        if (SceneManager._scene.constructor.name == "Scene_Status") {
            const width = Graphics.width;
            const height = Graphics.height;
            this._contentsSpriteKeStfs.bitmap = new Bitmap(width, height);
        }
    };


    //- 追加コンテンツの破棄(コア追加)
    const _Window_Help_destroyContents = Window_Help.prototype.destroyContents;
    Window_Help.prototype.destroyContents = function() {
        //Window_Base.prototype.destroyContents.call(this);
        _Window_Help_destroyContents.apply(this);
        if (SceneManager._scene.constructor.name == "Scene_Status") {
            const contetnsEx = this._contentsSpriteKeStfs;
            if (contetnsEx && contetnsEx.bitmap) {
                contetnsEx.bitmap.destroy();
            }
        }
    };


    //- 追加コンテンツのクリア(コア追加)
    const _Window_Help_clear = Window_Help.prototype.clear;
    Window_Help.prototype.clear = function() {
        _Window_Help_clear.apply(this);
        if (SceneManager._scene.constructor.name == "Scene_Status") {
            const contetnsEx = this._contentsSpriteKeStfs;
            if (contetnsEx && contetnsEx.bitmap) {
                contetnsEx.bitmap.clear();
            }
        }
    };

    const _Window_Help_refresh = Window_Help.prototype.refresh;
    Window_Help.prototype.refresh = function() {
        _Window_Help_refresh.apply(this);
        if (SceneManager._scene.constructor.name == "Scene_Status") {
            const contetnsEx = this._contentsSpriteKeStfs;
            if (contetnsEx && contetnsEx.bitmap) {
                contetnsEx.bitmap.clear();
            }
        }
    };



    //==================================================
    //--  ステータス自由欄の作成
    //==================================================

    //- 自由欄の描画 追加(コア追加)
    const _Window_Status_drawBlock2 = Window_Status.prototype.drawBlock2;
    Window_Status.prototype.drawBlock2 = function() {
        _Window_Status_drawBlock2.apply(this);
        // ステータス自由欄の描画
        drawFreeSpaceStatus(this, this._actor);
    };


    //- 経験値表示を消去(再定義)
    const _Window_Status_drawExpInfo = Window_Status.prototype.drawExpInfo;
    Window_Status.prototype.drawExpInfo = function(x, y) {
        // 自由欄テキストの取得
        if (getFreeText(this._actor)) { return; };  
        _Window_Status_drawExpInfo.apply(this, arguments);
    };


    //- ステータス自由欄の描画
    function drawFreeSpaceStatus(windo, actor) {
        const layout = keke_freeSpaceStatus;
        const fontSize = getFontSize(layout["文字サイズ"]);
        const outWidth = layout["文字縁取り幅"] || windo.contents.outlineWidth;;
        // 自由欄テキストの取得
        const text = getFreeText(actor, fontSize);
        if (!text) { return; }
        // コンテンツ取得 & クリア
        const contents = windo.contents;
        const contentsEx = windo._contentsSpriteKeStfs.bitmap;
        contentsEx.clear();
        // コンテンツを入れ替え
        windo.contents = contentsEx;
        // フォント設定の構築
        windo._fontCfgKeStfs = buildFontCfg(contents, { fontSize:fontSize, textColor:layout["文字色"], outWidth:outWidth });
        // 描画
        const padding = windo._padding;
        const x = layout["位置X"];
        const y = layout["位置Y"] + padding;
        const width = windo.width - x;
        const height = windo.height;
        // バック描画
        if (layout["バック表示"]) {
            const color = layout["バック色"] || "rgba(0, 0, 0, 0)";
            const round = layout["バック丸み"] || 0;
            const marginX = layout["バック余白X"] || 0;
            const marginY = layout["バック余白Y"] || 0;
            const bx = x - marginX - outWidth;
            const by = padding / 2 - marginY;
            fillSquare(contentsEx, bx, by, width + marginX * 2 - padding / 2 + outWidth, height + marginY * 2 - padding, color, 0, round);
        }
        // テキスト描画
        windo.drawTextEx(text, x, y, width);
        // フォント設定を消去
        windo._fontCfgKeStfs = null;
        // コンテンツを戻す
        windo.contents = contents;
    };


    //- 構築したフォント設定を適用(コア追加)
    const _Window_Status_resetFontSettings = Window_Status.prototype.resetFontSettings;
    Window_Status.prototype.resetFontSettings = function() {
        Window_Base.prototype.resetFontSettings.call(this);
        _Window_Status_resetFontSettings.apply(this);
        if (this._fontCfgKeStfs) {
            // フォント設定の適用
            applyFontCfg(this.contents, this._fontCfgKeStfs);
        }
    };



    //==================================================
    //--  ステータスヘルプ
    //==================================================

    //- ステータスヘルプの表示 呼び出し(コア追加)
    const _Scene_Status_refreshActor = Scene_Status.prototype.refreshActor;
    Scene_Status.prototype.refreshActor = function() {
        _Scene_Status_refreshActor.apply(this);
        // ステータスヘルプの表示
        showStatusHelp(this);
    };


    //- ステータスヘルプの表示
    function showStatusHelp(scene) {
        const layout = keke_StatusHelp;
        const fontSize = getFontSize(layout["文字サイズ"]);
        // フリーヘルプの取得
        const text = getFreeHelp(scene.actor(), fontSize);
        if (!text) { return; }
        const windo = scene._profileWindow;
        // コンテンツ取得 & クリア
        const contents = windo.contents;
        const contentsEx = windo._contentsSpriteKeStfs.bitmap;
        contents.clear();
        contentsEx.clear();
        // コンテンツを入れ替え
        windo.contents = contentsEx;
        // フォント設定の構築
        windo._fontCfgKeStfs = buildFontCfg(contents, { fontSize:fontSize, textColor:layout["文字色"], outWidth:layout["文字縁取り幅"]});
        // 描画
        const padding = windo._padding;
        const x = layout["位置X"] + padding;
        const y = layout["位置Y"] + padding;
        const width = windo.width;
        const height = windo.height;
        // バック描画
        if (layout["バック表示"]) {
            const color = layout["バック色"] || "rgba(0, 0, 0, 0)";
            const round = layout["バック丸み"] || 0;
            const marginX = layout["バック余白X"] || 0;
            const marginY = layout["バック余白Y"] || 0;
            const bx = padding / 2 - marginX;
            const by = padding / 2 - marginY;
            fillSquare(contentsEx, bx, by, width + marginX * 2 - padding, height + marginY * 2 - padding, color, 0, round);
        }
        // テキスト描画
        windo.drawTextEx(text, x, y, width);
        // フォント設定を消去
        windo._fontCfgKeStfs = null;
        // コンテンツを戻す
        windo.contents = contents;
    };


    //- 構築したフォント設定を適用(コア追加)
    const _Window_Help_resetFontSettings = Window_Help.prototype.resetFontSettings;
    Window_Help.prototype.resetFontSettings = function() {
        Window_Base.prototype.resetFontSettings.call(this);
        _Window_Help_resetFontSettings.apply(this);
        if (this._fontCfgKeStfs) {
            // フォント設定の適用
            applyFontCfg(this.contents, this._fontCfgKeStfs);
        }
    };



    //==================================================
    //--  一言メッセージウインドウ
    //==================================================

    function Window_ShortMessageKe() {
        this.initialize(...arguments);
    }
    
    Window_ShortMessageKe.prototype = Object.create(Window_Base.prototype);
    Window_ShortMessageKe.prototype.constructor = Window_ShortMessageKe;


    //- 初期化
    Window_ShortMessageKe.prototype.initialize = function(rect, text = "") {
        Window_Base.prototype.initialize.call(this, rect);
        // メッセージの描画
        this.drawMessage(text);
    };


    //- 追加コンテンツスプライトの形成
    Window_ShortMessageKe.prototype._createContentsSprite = function() {
        this._contentsSpriteKeStfs = new Sprite();
        this.addChild(this._contentsSpriteKeStfs);
        Window.prototype._createContentsSprite.call(this);
    };


    //- 追加コンテンツスプライトの更新
    Window_ShortMessageKe.prototype._updateContents = function() {
        Window.prototype._updateContents.call(this);
        const bitmap = this._contentsSpriteKeStfs.bitmap;
        if (bitmap) {
            this._contentsSpriteKeStfs.setFrame(0, 0, bitmap.width, bitmap.height);
        }
    };


    //- 追加コンテンツの形成
    Window_ShortMessageKe.prototype.createContents = function() {
        Window_Base.prototype.createContents.call(this);
        const width = Graphics.width;
        const height = Graphics.height;
        this._contentsSpriteKeStfs.bitmap = new Bitmap(width, height);
    };


    //- 追加コンテンツの破棄
    Window_ShortMessageKe.prototype.destroyContents = function() {
        Window_Base.prototype.destroyContents.call(this);
        const contetnsEx = this._contentsSpriteKeStfs;
        if (contetnsEx && contetnsEx.bitmap) {
            contetnsEx.bitmap.destroy();
        }
    };


    //- 一言メッセージの描画
    Window_ShortMessageKe.prototype.drawMessage = function(text) {
        const layout = keke_shortMessage;
        let fontSize = getFontSize(layout["文字サイズ"]);
        // コンテンツ取得 & クリア
        const contents = this.contents;
        const contentsEx = this._contentsSpriteKeStfs.bitmap;
        contents.clear();
        contentsEx.clear();
        // 位置
        const padding = this._padding;
        const x = layout["位置X"] + padding;
        const y = layout["位置Y"] + padding;
        const width = this.width - 12;
        const height = this.height;
        // 制御文字の処理
        const r = this.processControlChar(text);
        text = r.text;
        fontSize = r.fontSize || fontSize;
        textColor = r.textColor || layout["文字色"];
        outWidth = r.outWidth || layout["文字縁取り幅"];
        // フォント設定の構築
        this._fontCfgKeStfs = buildFontCfg(this.contents, { fontSize:fontSize, textColor:textColor, outWidth:outWidth});
        // コンテンツを入れ替え
        this.contents = contentsEx;
        // バック描画
        if (layout["バック表示"]) {
            const color = layout["バック色"] || "rgba(0, 0, 0, 0)";
            const round = layout["バック丸み"] || 0;
            const marginX = layout["バック余白X"] || 0;
            const marginY = layout["バック余白Y"] || 0;
            const bx = padding / 2 - marginX;
            const by = padding / 2 - marginY;
            fillSquare(contentsEx, bx, by, width + marginX * 2 - padding, height + marginY * 2 - padding, color, 0, round);
        }
        // テキスト描画
        this.resetFontSettings();
        const texts = text.split(/\n|\\n/);
        const lineSpace = layout["行間"] || 0;
        let drawY = y;
        texts.forEach(tx => {
            this.drawText(tx, x, drawY, width - padding);
            drawY += fontSize + lineSpace;
        });
        // コンテンツを戻す
        this.contents = contents;
    };


    //- 制御文字の処理
    Window_ShortMessageKe.prototype.processControlChar = function(text) {
        //text = text.replace(/\n/g, "");
        let fontSize = null;
        let textColor = null;
        let outWidth = null;
        // 文字サイズ
        let regExp = /\\fs\[([\d\+\-]*)\]/g;
        while (true) {
            const match = regExp.exec(text);
            if (!match) { break; }
            const str = match[0];
            fontSize = getFontSize(match[1]);
            const a = text.slice(0, regExp.lastIndex - str.length);
            const b = text.slice(regExp.lastIndex);
            text = a + b;
        }
        // 文字色
        regExp = /\\c\[(\d*)\]/g;
        while (true) {
            const match = regExp.exec(text);
            if (!match) { break; }
            const str = match[0];
            textColor = ColorManager.textColor(Number(match[1]));
            const a = text.slice(0, regExp.lastIndex - str.length);
            const b = text.slice(regExp.lastIndex);
            text = a + b;
        }
        // 縁取り幅
        regExp = /\\ow\[(\d*)\]/g;
        while (true) {
            const match = regExp.exec(text);
            if (!match) { break; }
            const str = match[0];
            outWidth = Number(match[1]);
            const a = text.slice(0, regExp.lastIndex - str.length);
            const b = text.slice(regExp.lastIndex);
            text = a + b;
        }
        return { text:text, fontSize:fontSize, textColor:textColor, outWidth:outWidth }
    };


    //- 構築したフォント設定を適用(コア追加)
    Window_ShortMessageKe.prototype.resetFontSettings = function() {
        Window_Base.prototype.resetFontSettings.call(this);
        if (this._fontCfgKeStfs) {
            // フォント設定の適用
            applyFontCfg(this.contents, this._fontCfgKeStfs);
        }
    };


    //- UIボタンエリアの高さ
    const _Scene_Base_buttonAreaHeight = Scene_Base.prototype.buttonAreaHeight;
    Scene_Base.prototype.buttonAreaHeight = function() {
        let h = _Scene_Base_buttonAreaHeight.apply(this);
        if (isValidShortMessage(this)) {
            const areaH = keke_shortMessage["UIボタンエリア高さ"];
            if (areaH) {
                return areaH;
            }
        }
        return h;
    };



    //==================================================
    //--  一言メッセージ
    //==================================================

    //- 一言メッセージの形成 呼び出し(コア追加)
    const _Scene_MenuBase_createButtons = Scene_MenuBase.prototype.createButtons;
    Scene_MenuBase.prototype.createButtons = function() {
        _Scene_MenuBase_createButtons.apply(this);
        // 一言メッセージの形成
        if (isValidShortMessage(this)) {
            createShortMessage(this);
        }
    };


    //- 一言メッセージの形成
    function createShortMessage(scene) {
        if (!isValidShortMessage(scene)) { return; }
        // キャンセルボタン/ページボタン
        const cancelButton = scene._cancelButton;
        const pageupButton = scene._pageupButton;
        const pagedownButton = scene._pagedownButton;
        // UIボタン幅
        const padding = 12;
        const cancelW = cancelButton ? cancelButton.width + padding : 0;
        const pageW = pageupButton ? (pageupButton.width + pagedownButton.width + 8) : 0;
        // ウインドウ位置/サイズ
        const rect = {};
        rect.x = (pageupButton ? pageW : 0);
        rect.y = scene.buttonAreaTop();
        rect.width = Graphics.width - cancelW - pageW;
        rect.height = scene.buttonAreaHeight();
        // 一言メッセージの取得
        const text = getShortMessage();
        if (!text) { return; }
        // ウインドウ形成
        this._shortMessageWindowKe = new Window_ShortMessageKe(rect, text);
        // チルド
        scene.addWindow(this._shortMessageWindowKe);
    };


    //- 一言メッセージ有効か
    function isValidShortMessage(scene) {
        if (isHideShortMaeeageScene(scene)) { return false; }
        if (!getShortMessage()) { return false; }
        return true;
    };


    //- 一言メッセージ表示しないシーンか
    function isHideShortMaeeageScene(scene) {
        // 表示しないシーン
        const hideScenes = keke_shortMessage["表示しないシーン"];
        if (!hideScenes || !hideScenes.length) { return false; }
        const currentScene = scene.constructor.name;
        for (const name of hideScenes) {
            const hideScene = name.match(/^Scene_/i) ? name : "Scene_" + name;
            if (currentScene == hideScene) { return true;}
        };
        // メインメニューのみ
        if (keke_shortMessage["メインメニューのみ"]) {
            if (scene.constructor.name != "Scene_Menu") { return true; }
        }
        return false;
    };


    //- 一言メッセージの取得
    function getShortMessage() {
        // 条件に合うメッセージを全て取得
        const messages = keke_shortMessageMaking.filter(d => {
            let condition = d["条件"];
            if (condition == null) { return true; }
            condition = (typeof condition == "string") ? condition.replace(/[\x1b\\]/g, "") : condition;
            return newFunc(condition, {}, $gameVariables._data, $gameSwitches._data);
        });
        if (!messages.length) { return; }
        // 表示順を取得
        const order = keke_shortMessage["表示順"] || "リスト順";
        let text = "";
        // リスト順
        if (order == "リスト順") {
            // 番号を取得
            if (!$gameSystem._shortMessageNumberKeStfs) { $gameSystem._shortMessageNumberKeStfs = 0; }
            if (!messages[$gameSystem._shortMessageNumberKeStfs]) { $gameSystem._shortMessageNumberKeStfs = 0; }
            const number = $gameSystem._shortMessageNumberKeStfs;
            text = getText(messages[number]["呼ぶテキスト"]);
        // ランダム
        } else {
            text = getText(messages[Math.randomInt(messages.length)]["呼ぶテキスト"]);
        }
        return text;
    };


    //- メニューを閉じたときに一言メッセージを進める
    const _Scene_Map_initialize = Scene_Map.prototype.initialize;
    Scene_Map.prototype.initialize = function() {
        _Scene_Map_initialize.apply(this);
        if (SceneManager._scene instanceof Scene_MenuBase) {
            if ($gameSystem._shortMessageNumberKeStfs == null) { $gameSystem._shortMessageNumberKeStfs = 0; } else
            { $gameSystem._shortMessageNumberKeStfs++; }
        }
    };



    //==================================================
    //--  自由欄パック
    //==================================================

    //- 自由欄テキストの取得
    function getFreeText(actor, fontSize) {
        // 自由欄パックの取得
        const pack = getFreePack(actor);
        if (!pack) { return; }
        // テキスト群を取得
        const textList = pack["自由欄テキスト"];
        if (!textList || !textList.length) { return; }
        // テキスト名の取得
        const textName = getTextName(textList, actor);
        // テキストの取得
        const text = getText(textName);
        return convertText(text, fontSize, actor);
    };


    //- フリーヘルプの取得
    function getFreeHelp(actor, fontSize) {
        // 自由欄パックの取得
        const pack = getFreePack(actor);
        if (!pack) { return; }
        // テキスト群を取得
        const textList = pack["ヘルプテキスト"];
        if (!textList || !textList.length) { return; }
        // テキスト名の取得
        const textName = getTextName(textList, actor);
        // テキストの取得
        const text = getText(textName);
        return convertText(text, fontSize, actor);
    };


    //- 自由欄パックの取得
    function getFreePack(actor) {
        // パック名を取得
        const names = totalAllMetaArray(actor, ["ステータス自由欄", "statusFreeSpace"]);
        if (!names || !names.length) { return; }
        const name = names[names.length - 1];
        if (!name) { return; }
        // パックを取得
        const pack = keke_freePackMaking.find(p => p["パック名"] == name);
        return pack;
    };


    //- テキスト名の取得
    function getTextName(textList, actor) {
        // 条件に合うテキストデータを取得
        const textData = textList.find(d => {
            let condition = d["条件"];
            if (condition == null) { return true; }
            condition = (typeof condition == "string") ? condition.replace(/[\x1b\\]/g, "") : condition;
            return newFunc(condition, actor, $gameVariables._data, $gameSwitches._data);
        });
        if (!textData) { return ""; }
        // テキスト名を取得
        return textData["呼ぶテキスト"] || "";
    };


    //- テキストの取得
    function getText(textName) {
        const textData = keke_textMaking.find(d => d["テキスト名"] == textName);
        if (!textData) { return ""; }
        return textData["テキスト"] || "";
    };


    //- テキストの置換
    function convertText(text, fontSize, actor) {
        const mainSize = fontSize || $gameSystem.mainFontSize();
        // フォントクリア
        text = text.replace(/\\cl/g, `\\c[0]\\fs[${mainSize}]`);
        // 文字サイズ+-
        let regExp = /\\fs\[([+-]+)(\d+)\]/g;
        while (true) {
            const match = regExp.exec(text);
            if (!match) { break; }
            const str = match[0];
            const symbol = match[1];
            const val = Number(match[2]);
            const size = fontSize + (symbol.includes("+") ? val : -val);
            const a = text.slice(0, regExp.lastIndex - str.length);
            const b = `\\fs[${size}]`;
            const c = text.slice(regExp.lastIndex);
            text = a + b + c;
        }
        // スクリプト
        regExp = /\\sc\[([^\]]+)\]/g;
        while (true) {
            const match = regExp.exec(text);
            if (!match) { break; }
            const str = match[0];
            const code = newFunc(match[1], actor, $gameVariables._data, $gameSwitches._data);
            const a = text.slice(0, regExp.lastIndex - str.length);
            const b = code;
            const c = text.slice(regExp.lastIndex);
            text = a + b + c;
        }
        return text;
    };


    //- ニューファンク
    let funcs = {};
    function newFunc(str, a, v, s) {
        if (!funcs[str]) {
            funcs[str] = new Function("a", "v", "s", "return " + str);
            return funcs[str](a, v, s);
        } else {
            return funcs[str](a, v, s);
        }
    };



    //==================================================
    //-- フォント基本 /ベーシック
    //==================================================

    //- フォントサイズの取得
    function getFontSize(size, oriSize) {
        if (!oriSize) { oriSize = $gameSystem.mainFontSize(); }
        if (!size) { return oriSize; }
        const sizeStr = size.toString();
        if (sizeStr.includes("+")) {
            const plus = Number(sizeStr.replace("+", ""));
            size = oriSize + plus;
        } else if (sizeStr.includes("-")) {
            const minus = Number(sizeStr.replace("-", ""));
            size = oriSize - minus;
        }
        return Number(size);
    };


     //- フォント設定の構築
     function buildFontCfg(contents, d = {}) {
        const cfg = {};
        cfg.fontFace = d.fontFace || contents.fontFace;
        cfg.fontSize = d.fontSize || contents.fontSize;
        cfg.textColor = d.textColor || contents.textColor;
        cfg.outlineWidth = d.outWidth || contents.outlineWidth;
        cfg.outlineColor = d.outColor || contents.outlineColor;
        return cfg;
    };


    //- フォント設定の適用
    function applyFontCfg(bitmap, cfg) {
        bitmap.fontFace = cfg.fontFace || bitmap.fontFace;
        bitmap.fontSize = cfg.fontSize || bitmap.fontSize;
        bitmap.textColor = cfg.textColor || bitmap.textColor;
        bitmap.outlineWidth = cfg.outlineWidth || bitmap.outlineWidth;
        bitmap.outlineColor = cfg.outlineColor || bitmap.outlineColor;
    };



    //==================================================
    //--  メタ取得 /ベーシック
    //==================================================
    
    //- 全てのメタ配列の合算
    function totalAllMetaArray(battler, words, action, noDelSpace) {
        let data = null
        let array = [];
        // バトラー値
        data = battler._actorId ? battler.actor() : battler.enemy();
        if (data) { metaAll(data.note, words).forEach(e => array.push(e)); }
        if (battler._actorId) {
            // 職業値
            data = battler.currentClass();
            if (data) { metaAll(data.note, words).forEach(e => array.push(e)); }
            // 装備値
            battler._equips.forEach(equip => {
                data = equip.object();
                if (data) { metaAll(data.note, words).forEach(e => array.push(e)); }
            });
        }
        // ステート値
        battler._states.forEach(stateId => {
            data = $dataStates[stateId];
            if (data) { metaAll(data.note, words).forEach(e => array.push(e)); }
        });
        // アクション値
        if (action) {
            data = action.item();
            if (data) { metaAll(data.note, words).forEach(e => array.push(e)); }
        }
        // スペース削除
        if (!noDelSpace) { array = array.map(e => e.replace(/\s/g, "")); }
        // 空の要素は削除
        array = array.filter(e => e);
        return array;
    };
    
    
    //- 全取得メタ
    function metaAll(note, words) {
        var result = [];
        words.forEach(word => {
            var regText = '\<' + word + ':([^\>]*)\>';
            var regExp_g = new RegExp(regText, 'ig');
            var regExp = new RegExp(regText);
            var matches = note.match(regExp_g);
            if (matches) {
                matches.forEach(function(line) {
                    const match = line.match(regExp);
                    result.push(match[1]);
                });
            }
        });
        return result;
    };



    //==================================================
    //--  図形描画 /ベーシック
    //==================================================
    
    //- スクエアの塗り潰し
    function fillSquare(bitmap, x, y, w, h, color = "rgba(0,0,0,1)", roundLine = 0, roundEdge = 0, corner = "") {
        const context = bitmap.context;
        context.save();
        if (Array.isArray(color)) {
            const x1 = x + w;
            const y1 = y + h;
            const grad = context.createLinearGradient(x, y, x1, y1);
            grad.addColorStop(0, color[0]);
            grad.addColorStop(1, color[1]);
            context.fillStyle = grad;
        } else {
            context.fillStyle = color;
        }
        designSquare(bitmap, x, y, w, h, roundLine, roundEdge, corner);
        context.fill();
        context.restore();
        bitmap._baseTexture.update();
    };
    
    
    //- スクエアの線画
    function strokeSquare(bitmap, x, y, w, h, color = "rgba(0,0,0,1)", lineW = 1,  roundLine = 0, roundEdge = 0, corner = "") {
        const context = bitmap.context;
        context.strokeStyle = color;
        context.lineWidth = lineW;
        designSquare(bitmap, x, y, w, h, roundLine, roundEdge, corner, lineW);
        context.stroke();
        context.restore();
        bitmap._baseTexture.update();
    };
    
    
    //- スクエアのデザイン
    function designSquare(bitmap, x, y, w, h, roundLine = 0, roundEdge = 0, corner = "", lineW = 0, airUp) {
        const context = bitmap.context;
        context.beginPath();
        const x2 = x + w / 2;
        const y2 = y + h / 2;
        const x3 = x + w;
        const y3 = y + h;
        const c1 = corner.includes("1") ? 0 : roundEdge;    // 左上
        const c2 = corner.includes("2") ? 0 : roundEdge;    // 左下
        const c3 = corner.includes("3") ? 0 : roundEdge;    // 右上
        const c4 = corner.includes("4") ? 0 : roundEdge;    // 右下
        context.moveTo(x + c1,  y);
        // 右上
        if (roundLine && !airUp) {
            context.quadraticCurveTo(x2,  y + roundLine,  x3 - c3,  y);
        } else {
            context.lineTo(x3 - c3,  y);
        }
        context.quadraticCurveTo(x3,  y,  x3,  y + c3);
        // 右下
        if (roundLine) {
            context.quadraticCurveTo(x3 - roundLine,  y2,  x3,  y3 - c4);
        } else {
            context.lineTo(x3,  y3 - c4);
        }
        context.quadraticCurveTo(x3,  y3,  x3 - c4,  y3);
        // 左下
        if (roundLine) {
            context.quadraticCurveTo(x2,  y3 - roundLine,  x + c2,  y3);
        } else {
            context.lineTo(x + c2,  y3);
        }
        context.quadraticCurveTo(x,  y3,  x,  y3 - c2);
        // 左上
        if (roundLine) {
            context.quadraticCurveTo(x + roundLine,  y2,  x,  y + c1);
        } else {
            context.lineTo(x,  y + c1);
        }
        if (c1) {
            context.quadraticCurveTo(x,  y,  x + c1,  y);
        } else {
            context.lineTo(x,  y - lineW / 2);
        }
    };
    
})();